/*
 * Copyright (c) 2003-onwards Shaven Puppy Ltd
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'Shaven Puppy' nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package worm.animation;

import worm.features.LayersFeature;

import com.shavenpuppy.jglib.sprites.Sprite;
import com.shavenpuppy.jglib.sprites.SpriteAllocator;

/**
 * Convenient holder for layers of sprites
 */
public class SimpleThingWithLayers implements ThingWithLayers {

	private static final long serialVersionUID = 1L;

	private final SpriteAllocator allocator;

	private Sprite[] sprite;
	private LayersFeature appearance;

	/**
	 * C'tor
	 */
	public SimpleThingWithLayers(SpriteAllocator allocator) {
		this.allocator = allocator;
	}

	@Override
	public void requestSetAppearance(LayersFeature newAppearance) {
		if (appearance == newAppearance) {
			return;
		}
		appearance = newAppearance;
		if (appearance != null) {
			appearance.createSprites(allocator, this);
		} else {
			removeSprites();
		}
	}

	@Override
	public LayersFeature getAppearance() {
		return appearance;
	}

	@Override
	public Sprite getSprite(int idx) {
		return sprite[idx];
	}

	public Sprite[] getSprites() {
		return sprite;
	}

	@Override
	public void setSprites(Sprite[] newSprites) {
		if (sprite != null) {
			removeSprites();
		}
		sprite = newSprites;
	}

	@Override
	public void remove() {
		if (sprite != null) {
			removeSprites();
		}
	}

	private void removeSprites() {
		for (Sprite element : sprite) {
			if (element != null) {
				element.deallocate();
			}
		}
		sprite = null;
	}

}
