/*
 * Copyright (c) 2003-onwards Shaven Puppy Ltd
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'Shaven Puppy' nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package worm.features;

import net.puppygames.applet.effects.EmitterFeature;

import org.lwjgl.util.Point;

import com.shavenpuppy.jglib.openal.ALBuffer;
import com.shavenpuppy.jglib.opengl.GLTexture;
import com.shavenpuppy.jglib.resources.Feature;

/**
 * Bombs wot are dropped from flying gidrahs
 */
public class BombFeature extends Feature {

	private static final long serialVersionUID = 1L;

	private String explosionTexture;
	private float explosionRadius;
	private int explosionDamage;
	private int shake;
	private String dropNoise;
	private LayersFeature appearance;
	private EmitterFeature explosion;
	private Point bombOffset, mirroredBombOffset;

	private transient GLTexture explosionTextureResource;
	private transient ALBuffer dropNoiseResource;

	/**
	 * C'tor
	 */
	public BombFeature() {
	}

	/**
	 * C'tor
	 * @param name
	 */
	public BombFeature(String name) {
		super(name);
	}

	/**
	 * @return the bombOffset
	 */
	public Point getBombOffset(boolean mirrored) {
		return mirrored ? mirroredBombOffset : bombOffset;
	}

	/**
	 * @return the dropNoiseResource
	 */
	public ALBuffer getDropNoise() {
		return dropNoiseResource;
	}

	/**
	 * @return the appearance
	 */
	public LayersFeature getAppearance() {
		return appearance;
	}

	/**
	 * @return the explosionRadius
	 */
	public float getExplosionRadius() {
		return explosionRadius;
	}

	/**
	 * @return the explosionDamage
	 */
	public int getExplosionDamage() {
		return explosionDamage;
	}

	/**
	 * @return the shake
	 */
	public int getShake() {
		return shake;
	}

	/**
	 * @return the explosion
	 */
	public EmitterFeature getExplosion() {
		return explosion;
	}

	/**
	 * @return the explosionTextureResource
	 */
	public GLTexture getExplosionTexture() {
		return explosionTextureResource;
	}

}
